@@ -8,7 +8,7 @@ import cvxopt.glpk
 from scipy.optimize import linprog
 
 
-def iterative_convex_hull_method(A, B, y_min, y_max, tol, P = None, bias = None,  G_in = None, h_in = None, G_eq = None, h_eq = None):
+def iterative_convex_hull_method(A, B, y_min, y_max, tol, P = None, bias = None,  G_in = None, h_in = None, G_eq = None, h_eq = None, max_iter=1000):
     """
     A function calculating the polytopes of achievable x for equations form:
 
@@ -32,11 +32,11 @@ def iterative_convex_hull_method(A, B, y_min, y_max, tol, P = None, bias = None,
 
     (optionally - additional inequality constaints)
 
-    .. math:: G_{in} x \leq h_{in}    
+    .. math:: G_{in} y \leq h_{in}    
     
     (optionally - additional equality constaints)
 
-    .. math:: G_{eq} x = h_{eq}
+    .. math:: G_{eq} y = h_{eq}
 
     Note:
         On-line feasible wrench polytope evaluation based on human musculoskeletal models: an iterative convex hull method
@@ -50,10 +50,11 @@ def iterative_convex_hull_method(A, B, y_min, y_max, tol, P = None, bias = None,
         tol: tolerance for the polytope calculation
         P: an additional projection matrix 
         bias: bias in the intermediate space 
-        G_in: matrix - inegality constraint G_in x < h_in
-        h_in: vector - inegality constraint G_in x < h_in
-        G_eq: matrix - equality constraint G_eq x = h_eq
-        h_eq: vector - equality constraint G_eq x = h_eq
+        G_in: matrix - inegality constraint G_in y < h_in
+        h_in: vector - inegality constraint G_in y < h_in
+        G_eq: matrix - equality constraint G_eq y = h_eq
+        h_eq: vector - equality constraint G_eq y = h_eq
+        max_iter: maximum number of iterations (number of linera programmes solved) - default 1000
     
     Returns
     ---------
@@ -153,12 +154,8 @@ def iterative_convex_hull_method(A, B, y_min, y_max, tol, P = None, bias = None,
         # check the size
         nG, mG = G_eq.shape
         if mG != L:
-            raise ValueError('Matrix G_in dimensions error - (rows,cols) = G_in.shape should be: col = {:d}. In your case Geq.shape = {}.'.format(L,G_eq.shape))
-        if nG != len(h_eq):
-            raise ValueError('Vector h_in dimensions error - should have {:d} entries. In your case h_eq.shape = {}.'.format(nG,len(h_eq)))    
-        if Aeq is not None:
-            Aeq = matrix(np.vstack((Aeq,G_eq)))
-            beq = matrix(np.hstack((beq,h_eq)))
+            raise ValueEr
+            # raise error here insteadnp.hstack((beq,h_eq)))
         else:
             Aeq = matrix(G_eq)
             beq = matrix(h_eq)
@@ -168,10 +165,13 @@ def iterative_convex_hull_method(A, B, y_min, y_max, tol, P = None, bias = None,
     #solvers.options['glpk'] = dict(msg_lev='GLP_MSG_OFF')
     # solvers_opt.options['solver'] = 'mosek' # 'glpk'
 
+    linprog_count = 0
+
     solvers_opt={'tm_lim': 100000, 'msg_lev': 'GLP_MSG_OFF', 'it_lim':10000}
 
     y_vert, x_p = [],[]
     for i in range(m):
+        linprog_count = linprog_count + 2
         c = matrix((u[:,i].T).dot(M))
         res = cvxopt.glpk.lp(c=-c,  A=Aeq, b=beq, G=G,h=h, options=solvers_opt)
         if res[1]:
@@ -193,6 +193,7 @@ def iterative_convex_hull_method(A, B, y_min, y_max, tol, P = None, bias = None,
         try:
             hull = ConvexHull(x_p.T, incremental=True, qhull_options="QJ")
         except:
+            print("ICHM: Search stopped prematurely - inital convex hull not found!")
             z_vert = B.dot(y_vert) + bias
             x_vert  = M.dot(y_vert) + x_bias
             return x_vert, [], [], [], [], []
@@ -200,22 +201,30 @@ def iterative_convex_hull_method(A, B, y_min, y_max, tol, P = None, bias = None,
     n_faces, n_faces_old,  = len(hull.simplices), 0
     face_final, cnt = {}, 2*m
 
-    while n_faces > n_faces_old:
-        n_faces_old = n_faces
+
+    max_delta = tol*100
+    # iterate until the maxima
+            # raise error here insteadl distance between the target and 
+    # the aproximated polytope is under tol value
+    while max_delta > tol and linprog_count <= max_iter:
         
         x_center = np.mean(x_p,axis=1)
         
         y_vert_new = []
 
+        max_delta = 0
         for face, equation in zip(hull.simplices,hull.equations):
             
             # create a string index of the face 
             # this value is used as a hash map index
             # to store dyamically the faces that have been found as final
-            face_key = str(np.sort(face))
+            face_key = str(np.sort(equation))
             # check if this face (face index) has been found as final
             if face_key in face_final.keys():
                 continue; 
+            
+            # update linprog counter
+            linprog_count = linprog_count + 1
 
             # calculate the normal vector to the face
             face_normal = equation[:-1]
@@ -241,6 +250,9 @@ def iterative_convex_hull_method(A, B, y_min, y_max, tol, P = None, bias = None,
                 y_vert_new = stack(y_vert_new, res, 'h')
             else:
                 face_final[face_key] = 1
+            
+            if distance > max_delta:
+                max_delta = distance
 
         if len(y_vert_new):
             x_p_new = M.dot(y_vert_new) + x_bias
@@ -254,11 +266,15 @@ def iterative_convex_hull_method(A, B, y_min, y_max, tol, P = None, bias = None,
 
             z_new = B.dot(y_vert_new) + bias
             z_vert = stack(z_vert, z_new,'h')
- 
-        n_faces = len(hull.simplices)
-  
+        elif max_delta > tol: 
+            print("ICHM: Search stopped prematurely - search stuck at precision: {}!".format(max_delta))
+            # raise error here instead
+            break 
 
-    return x_p, hull.equations[:,:-1], hull.equations[:,-1], hull.simplices, y_vert, z_vert
+    if linprog_count >= max_iter:
+        print("ICHM: Max iteration number reached: {}".format(max_iter))
+
+    return hull.points.T, hull.equations[:,:-1], -hull.equations[:,-1], hull.simplices, y_vert, z_vert
 
 
 def hyper_plane_shift_method(A, x_min, x_max, tol = 1e-15):
@@ -447,6 +463,32 @@ def vertex_enumeration_auctus(A, b_max, b_min, b_bias = None):
     x_vertex = A_inv.dot( b_vertex )
     return x_vertex, b_vertex
 
+def chebyshev_center(A,b):
+    """
+    Calculating chebyshev center of a polytope given the half-space representation
+
+    https://pageperso.lis-lab.fr/~francois.denis/IAAM1/scipy-html-1.0.0/generated/scipy.spatial.HalfspaceIntersection.html
+
+    Args:
+        A(list):  
+            matrix of half-space representation `Ax<b`
+        b(list): 
+            vector of half-space representation `Ax<b`
+    Returns:
+        center(array): returns a chebyshev center of the polytope
+    """
+    # calculate the certices
+    Ab_mat = np.hstack((np.array(A),-np.array(b)))
+
+    # calculating chebyshev center
+    norm_vector = np.reshape(np.linalg.norm(A[:, :-1], axis=1), (A.shape[0], 1))
+    c = np.zeros((Ab_mat.shape[1],))
+    c[-1] = -1
+    G = matrix(np.hstack((Ab_mat[:, :-1], norm_vector)))
+    h = matrix(- Ab_mat[:, -1:])
+    solvers_opt={'tm_lim': 100000, 'msg_lev': 'GLP_MSG_OFF', 'it_lim':10000}
+    res = cvxopt.glpk.lp(c=c,  G=G, h=h, options=solvers_opt)
+    return np.array(res[1][:-1]).reshape((-1,))
 
 def hsapce_to_vertex(H,d):
     """
@@ -464,21 +506,10 @@ def hsapce_to_vertex(H,d):
 
     """
     if len(H):
-        # calculate the certices
-        hd_mat = np.hstack((np.array(H),-np.array(d)))
 
-        # calculating chebyshev center
-        # https://pageperso.lis-lab.fr/~francois.denis/IAAM1/scipy-html-1.0.0/generated/scipy.spatial.HalfspaceIntersection.html
-        norm_vector = np.reshape(np.linalg.norm(H[:, :-1], axis=1), (H.shape[0], 1))
-        c = np.zeros((hd_mat.shape[1],))
-        c[-1] = -1
-        A = np.hstack((hd_mat[:, :-1], norm_vector))
-        b = - hd_mat[:, -1:]
-        G = matrix(A)
-        h = matrix(b)
-        solvers_opt={'tm_lim': 100000, 'msg_lev': 'GLP_MSG_OFF', 'it_lim':10000}
-        res = cvxopt.glpk.lp(c=c,  G=G, h=h, options=solvers_opt)
-        feasible_point = np.array(res[1][:-1]).reshape((-1,))
+        hd_mat = np.hstack((np.array(H),-np.array(d)))
+        # calculate a feasible point inside the polytope
+        feasible_point = chebyshev_center(H,d)
 
         # calculate the convex hull
         hd = HalfspaceIntersection(hd_mat,feasible_point,'QJ')
@@ -486,6 +517,16 @@ def hsapce_to_vertex(H,d):
         return hd.intersections.T, hull.simplices
 
 def vertex_to_faces(vertex):
+    """
+    Function grouping the vertices to faces using a ConvexHull algorithm
+
+
+    Args:
+        vertex(array):  list of verteices
+
+    Returns:
+        faces(array) : list of triangle faces with vertex indexes which form them
+    """
     if vertex.shape[0] == 1:
         faces = [0, 1]
     else:        
